/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: DummyStoragePool.java 175 2011-07-21 14:22:52Z dangtran $
 *
 */

package org.ow2.sirocco.vmm.agent.driver.dummy;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.management.ObjectName;

import org.apache.log4j.Logger;
import org.ow2.sirocco.vmm.agent.domain.AbstractStoragePool;
import org.ow2.sirocco.vmm.api.InsufficientResourcesException;
import org.ow2.sirocco.vmm.api.StoragePoolMXBean;
import org.ow2.sirocco.vmm.api.VMMException;
import org.ow2.sirocco.vmm.api.Volume;

class DummyStoragePool extends AbstractStoragePool implements StoragePoolMXBean {
    static protected Logger logger = Logger.getLogger(DummyStoragePool.class);

    private CopyOnWriteArrayList<Volume> volumes;

    private int capacityMB;

    private int usedSpaceMB;

    private String name;

    private Map<String, String> properties;

    DummyStoragePool(final String name, final ObjectName objectName, final Map<String, String> properties) {
        super(objectName);
        this.name = name;
        this.volumes = new CopyOnWriteArrayList<Volume>();
        this.capacityMB = 10240;
        this.properties = new HashMap<String, String>(properties);
        this.volumes.add(this.newDummyVolume("dummy"));
        this.volumes.add(this.newDummyVolume("dummy"));
        this.volumes.add(this.newDummyVolume("iso"));
        this.volumes.add(this.newDummyVolume("iso"));
    }

    private int counter = 100;

    private Volume newDummyVolume(final String format) {
        Volume vol = new Volume();
        vol.setCapacityMB(512);
        vol.setFormat(format);
        vol.setKey("vol-" + this.counter++);
        vol.setPath(vol.getKey());
        vol.setName(vol.getKey());
        vol.setStoragePool(this);
        this.usedSpaceMB += vol.getCapacityMB();
        return vol;
    }

    @Override
    public String getName() {
        return this.name;
    }

    @Override
    public String[] getSupportedDiskImageFormats() {
        return new String[] {"dummy"};
    }

    @Override
    public Map<String, String> getProperties() {
        return this.properties;
    }

    @Override
    public void refresh() {
    }

    @Override
    public synchronized Volume createVolume(final String name, final long capacityMB, final String format) throws VMMException {
        DummyStoragePool.logger.debug("Creating volume name=" + name + " capacityMB=" + capacityMB + " format=" + format);
        Volume disk = new Volume();
        disk.setCapacityMB(capacityMB);
        disk.setStoragePool(this);
        disk.setName(name);
        disk.setKey(name);
        disk.setPath(disk.getKey());
        disk.setFormat("dummy");
        this.volumes.add(disk);
        this.usedSpaceMB += disk.getCapacityMB();
        return disk;
    }

    @Override
    public Volume createVolumeFrom(final String name, final long capacityMB, final String format, final Volume parent,
        final boolean copyOnWrite) throws VMMException {
        return this.createVolume(name, capacityMB, format);
    }

    @Override
    public synchronized void destroyVolume(final String key) throws VMMException {
        for (Volume disk : this.volumes) {
            if (disk.getKey().equals(key)) {
                this.volumes.remove(disk);
                this.usedSpaceMB -= disk.getCapacityMB();
                return;
            }
        }
        throw new VMMException("disk not found, uuid=" + key);
    }

    @Override
    public long getCapacityMB() {
        return this.capacityMB;
    }

    @Override
    public long getFreeSpaceMB() {
        return this.capacityMB - this.usedSpaceMB;
    }

    @Override
    public List<Volume> getVolumes() throws VMMException {
        return this.volumes;
    }

    @Override
    public String getType() {
        return "dummy";
    }

    @Override
    public Volume getVolumeByKey(final String key) {
        for (Volume disk : this.volumes) {
            if (disk.getKey().equals(key)) {
                return disk;
            }
        }
        return null;
    }

    @Override
    public Volume uploadVolume(final String url, final String format, final String name, final String description,
        final String checkSum) throws InsufficientResourcesException, VMMException {
        Volume disk = new Volume();
        disk.setCapacityMB(512);
        disk.setStoragePool(this);
        disk.setKey("vol-" + this.counter++);
        disk.setName(name);
        disk.setPath(disk.getKey());
        disk.setFormat(format);
        disk.setDescription(description);
        disk.setCheckSum(checkSum);
        this.volumes.add(disk);
        return disk;
    }
}
